unit Llibro;

  {Esta unidad contiene los procedimientos y funciones necesarios para
   dar de alta y eliminar libros, aparte de guardarlos e inicializar la
   BBDD. Toma prestados los tipos definidos en la unidad Libro}

interface

uses Libro;

{********************************************************************}

type

ArchivoLibros = file of BBDDLibros;

  {Archivo para guardar.}

{********************************************************************}

procedure CrearBBDDLibros(var Lib: BBDDLibros);

  {Inicializa la BBDD a 0.}

procedure GuardarLibros(Lib: BBDDLibros; NombreLib: string);

  {Guarda la BBDD en disco.}

procedure InsertarLibros(NombreLib: string; var Lib: BBDDLibros);

  {Inserta un libro en la BBDD.}

procedure EliminarLibros(NombreLib: string; var Lib: BBDDLibros);

  {Elimina libros de la BBDD.}

{********************************************************************}

implementation

procedure CrearBBDDLibros(var Lib: BBDDLibros);

  {Crea una BBDD vacia, y la devuelve}

begin
  Lib.Limite := 0;
end;

{********************************************************************}

function BBDDVacia(Lib: BBDDLibros): boolean;

  {Comprueba si la BBDD esta vacia, necesitandola para ello}

begin
  BBDDVacia := (Lib.Limite = 0)
end;

{********************************************************************}

procedure GuardarLibros(Lib: BBDDLibros; NombreLib: string);

  {Guarda los datos en disco. Precisa de la BBDD y el nombre del archivo.}

var
  ArchLib: file of BBDDLibros;

  {Archivo para la BBDD.}

begin
  assign(ArchLib, NombreLib);
  rewrite(ArchLib);
  write(ArchLib, Lib);
  close(ArchLib);
end;

{********************************************************************}

procedure InsertarLibros(NombreLib: string; var Lib: BBDDLibros);

  {Inserta nuevos libros en la BBDD, necesitando para ello el nombre
   del archivo para el autoguardado y la BBDD en s.}

  {******************************************************************}

  procedure PonOrden (RegisLib: RegistroDeLibros;
                      var Lib: BBDDLibros);

    {Ordena el registro recien leido del usuario, y lo coloca en la BBDD.}

  var

    i, coordenada: integer;

      {Una variable de recorrido y otra para almacenar la posicion en la
       que deberia ir el registro.}

  begin

    with Lib do
      begin
        if Limite = 0 then
          begin
            Limite := 1;
            Elemento[Limite] := RegisLib;
          end
        else
          begin
            for i := 2 to Limite do
              if (Elemento[i].signatura > RegisLib.signatura) and
                 (Elemento[i - 1].signatura < RegisLib.signatura) then
                coordenada := i;
            if (Elemento[Limite].signatura < RegisLib.signatura) then
              coordenada := Limite + 1;
            if (Elemento[1].signatura > RegisLib.signatura) then
              coordenada := 1;
            Limite := Limite + 1;
            for i := Limite downto (coordenada + 1) do
              Elemento[i] := Elemento[i - 1];
            Elemento[coordenada] := RegisLib;
          end
      end
  end;

  {******************************************************************}

var
  RegistroLib: RegistroDeLibros;

  {Registro temporal en el que guardaremos los datos hasta ser introducidos
   en la BBDD.}

  i: integer;

  {Variable de recorrido.}

  fallo, lleno: boolean;

  {Variables de control.}

  aux: char;

  {Variable para interpretar el estado del libro.}

begin

  lleno := false;

  if Lib.Limite = MaxLibros then
    begin
      lleno := true;
    end;

  if not lleno then
    with RegistroLib do
      begin
        write('Titulo del libro (. para salir): ');
        readln(titulo);
        while (titulo <> '.') do
          begin
            write('Autor: ');
            readln(autor);
            write('Edicion: ');
            readln(edicion);
            repeat
              fallo := false;
              write('Signatura: ');
              readln(signatura);
              for i := 1 to Lib.Limite do
                if Lib.Elemento[i].signatura = signatura then
                  begin
                    writeln;
                    writeln('Ya existe otro libro con esa signatura.');
                    writeln;
                    fallo := true;
                  end;
            until fallo = false;
            repeat
              fallo := false;
              write('(D)isponible o d(e)posito?: ');
              readln(aux);
              if (aux <> 'd') and (aux <> 'D') and (aux <> 'e') and
                 (aux <> 'E') then
                fallo := true;
              case aux of
                'd', 'D': situacion := Disponible;
                'e', 'E': situacion := Deposito;
              end;
            until fallo = false;
            PonOrden(RegistroLib, Lib);
            if Lib.Limite = MaxLibros then
              lleno := true;
            write('Titulo del libro (. para salir): ');
            readln(titulo);
          end;
          GuardarLibros(Lib, NombreLib);
          writeln;
          writeln('Entradas aadidas');
          readln;
      end;

  if lleno = true then
    begin
      writeln('La base de datos esta llena.');
      readln;
    end

end;

{********************************************************************}

procedure EliminarLibros (NombreLib: string; var Lib: BBDDLibros);

  {Este procedimiento elimina libros de la BBDD. Para ello necesita
   el nombre del archivo y devuelve la BBDD modificada. En caso de
   no existir el libro, o estar prestado, se muestran mensajes de
   error.}

var
  sig: string;

  {Signatura del libro.}

  i, coordenada: integer;

  {Variable de recorrido, y la que guarda la casilla a eliminar.}

  fallo, hallado: boolean;

  {Variables de control}

begin
  hallado := false;
  fallo := false;
  coordenada := 0;
  if not BBDDVacia(Lib) then
    begin
      writeln;
      write('Introduzca la signatura del libro a eliminar: ');
      readln(sig);
      with Lib do
        begin
          for i := 1 to Limite do
            begin
              if (sig = elemento[i].signatura) then
                begin
                  if (elemento[i].situacion <> Prestado) then
                    begin
                      coordenada := i;
                      hallado := true;
                    end
                  else
                    begin
                      fallo := true;
                      writeln('El libro esta en uso');
                    end;
                end;
              if (i = Limite) and (fallo = false) and (hallado = false) then
                begin
                  fallo := true;
                  writeln('El libro especificado no existe.');
                end
            end;
          if fallo = false then
            begin
              for i := coordenada to Limite do
                Elemento[i] := Elemento[i + 1];
              Limite := Limite - 1;
              GuardarLibros(Lib, NombreLib);
              writeln('Entradas borradas.');
            end;
        end
    end
  else
    writeln('Esta intentando eliminar un libro de una BBDD vacia.');

  readln;

end;

{********************************************************************}

end.