unit LSocio;

  {Esta unidad se encarga de llevar los registros de socios. Darlos de alta,
   eliminarlos, y guardarlos. Tambien define los tipos para las BBDD que
   se usan en el programa principal y la unidad Socio.}

interface

uses Socio;

  {Usa la unidad socio para definir los tipos de la BBDD de socios.}

{********************************************************************}

type

ArchivoSocios = file of BBDDSocios;

  {Tipo del archivo en el que guardar los datos.}

{********************************************************************}

procedure CrearBBDDSocios(var Soc: BBDDSocios);

  {Inicializa la BBDD a 0 socios.}

procedure GuardarSocios(Soc: BBDDSocios; NombreSoc: string);

  {Guarda la lista de socios en memoria en disco}

procedure InsertarSocios(NombreSoc: string; var Soc: BBDDSocios);

  {Inserta socios en memoria, y guarda al terminar}

procedure EliminarSocios(NombreSoc: string; var Soc: BBDDSocios);

  {Elimina socios en memoria, y guarda al terminar. Es indispensable
   que no tenga libros prestados antes de la eliminacion, o si no,
   no se permitira.}

{********************************************************************}

implementation

procedure CrearBBDDSocios(var Soc: BBDDSocios);

  {Crea una BBDD vacia, devuelve la BBDD.}

begin
  Soc.Limite := 0;
end;

{********************************************************************}

function BBDDVacia(Soc: BBDDSocios): boolean;

  {Comprueba si la BBDD esta vacia, devuelve un valor booleano con esta
   informacion.}

begin
  BBDDVacia := (Soc.Limite = 0)
end;

{********************************************************************}

procedure GuardarSocios(Soc: BBDDSocios; NombreSoc: string);

  {Este procedimiento guarda la lista de socios que hay en memoria en
   disco, necesitando para ello la BBDD y el nombre del archivo.}

var
  ArchSoc: ArchivoSocios;

  {Fichero binario en el que se guardar la BBDD.}

begin
  assign(ArchSoc, NombreSoc);
  rewrite(ArchSoc);
  write(ArchSoc, Soc);
  close(ArchSoc);
end;

{********************************************************************}

procedure InsertarSocios(NombreSoc: string; var Soc: BBDDSocios);

  {Inserta socios nuevos en la lista que se guarad en memoria, y al
   finalizar, se guarda automaticamente con la ultima configuracion
   conocida. Para ello necesita el nombre del archivo y devuelve la
   BBDD modificada.}

var
  RegistroSoc: RegistroDeSocios;

  {Registro temporal en el que se guardarn los datos del nuevo socio.}

  i: integer;

  {Variable de recorrido.}


  fallo, lleno: boolean;

  {Variables de control autoexplicativas.}

begin

  lleno := false;
  if Soc.Limite = MaxSocios then
    begin
      writeln('La base de datos esta llena.');
      readln;
    end
  else
    with RegistroSoc do
      begin
        write('Nombre del socio (. para salir): ');
        readln(nombre);
        while (nombre <> '.') and (lleno = false) do
          begin
            repeat
              fallo := false;
              write('Carnet: ');
              readln(carnet);
              for i := 1 to Soc.Limite do
                if Soc.Elemento[i].carnet = carnet then
                  begin
                    writeln;
                    writeln('Ya existe un socio con ese carnet');
                    writeln;
                    fallo := true;
                  end
            until fallo = false;
            Soc.Limite := Soc.Limite + 1;
            Soc.Elemento[Soc.Limite] := RegistroSoc;
            Soc.Elemento[Soc.Limite].Prestados.Limite := 0;
            if Soc.Limite = MaxSocios then
               begin
                 lleno := true;
                 writeln('La base de datos esta llena');
                 readln;
               end;
           if lleno = false then
             begin
               write('Nombre del socio (. para salir): ');
               readln(nombre);
             end
          end;
        GuardarSocios(Soc, NombreSoc);
        writeln;
        writeln('Socios dados de alta.');
        readln;
      end;
end;

{********************************************************************}

procedure EliminarSocios(NombreSoc: string; var Soc: BBDDSocios);

  {Elimina socios de la BBDD en memoria, y guarda en disco al finalizar.
   Para ello, comprueba si el socio tiene libros por devolver aun. Si
   tiene, no se puede eliminar, si no los tiene, se elimina sin problemas.
   Necesita el nombre del archivo y devuelve la BBDD.}

var
  car, i, coordenada: integer;

  {Car es la variable donde guardaremos el carnet del socio a eliminar.
   I es una variable de recorrido, y coordenada sera la casilla del array
   donde se encuentran los datos del socio a eliminar.}

  fallo: boolean;

  {Variable de control, para finalizar con exito la busqueda y mostrar un
   mensaje de error}

begin
  fallo := false;
  if not BBDDVacia(Soc) then
    begin
      writeln;
      write('Introduzca el carnet del socio a eliminar: ');
      readln(car);
      with Soc do
        begin
          for i := 1 to Limite do
            begin
              if car = elemento[i].carnet then
                coordenada := i;
              if (i = limite) and (coordenada = 0) then
                fallo := true;
            end;
          if fallo = true then
            begin
              writeln('El socio especificado no existe');
              readln;
            end
          else
            begin
              if elemento[coordenada].Prestados.Limite = 0 then
                begin
                  for i := coordenada to Limite do
                    Elemento[i] := Elemento[i + 1];
                  Limite := Limite - 1;
                  GuardarSocios(Soc, NombreSoc);
                  writeln('Socio eliminado.');
                  readln;
                end
              else
                begin
                  writeln('El usuario especificado aun tiene libros por devolver.');
                  readln;
                end
            end
        end
    end
  else
    begin
      writeln('Esta intentando eliminar un socio de una BBDD vacia.');
      readln;
    end
end;

{********************************************************************}

end.
